/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				interrupts.c
|
| Description:		Interrupt Handlers.
|
| Global Routines:	InitializeInterrupts
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <p18f458.h>    // Register definitions for the PIC18F458

#include "project.h"	// Includes for the project (Globals + misc)
#include "main.h"		// Includes for the mainline
#include "interrupts.h"	// Includes for interrupt routines and variables
#include "exceptions.h"	// Includes for exception handling
#include "ports.h"		// Includes for the port peripherals
#include "timer2.h"		// Includes for the Timer2 peripheral
#include "serial.h"		// Includes for the serial peripheral (UART )
//#include "can.h"		// Includes for the CAN peripheral


//------------------------------------------------------------------------
//  Local function prototypes
//------------------------------------------------------------------------

void LowIntVector (void);
void HighIntVector (void);

void LowPriorityISR (void);
void HighPriorityISR (void);



//------------------------------------------------------------------------
//  Create Interrupt Vectors (Manually, the bastards)
//------------------------------------------------------------------------

#pragma code lowVector=0x0018		// Force this code at address 0x18 (low priority IV)

void LowIntVector (void)
{
	_asm
		GOTO LowPriorityISR
	_endasm
}

#pragma code HighVector=0x0008		// Force this code at address 0x18 (low priority IV)

void HighIntVector (void)
{
	_asm
		GOTO HighPriorityISR
	_endasm
}

#pragma code 						// return to default code section


/*---------------------------------------------------------------------------\
| Function:		HighPriorityISR
|
| Purpose:		Handle all of the high priority interrupts
|
| Discussion:	Checks to see what interrupted us.
|
| Expects:		all interrupts to be disabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE HIGH PRIOIRTY INTERRUPT SERVICE ROUTINE
// USING THE #PRAGMA DIRECTIVE (ugly, but true).

#pragma interrupt HighPriorityISR	// Correct: marks a high prioirity interrupt

void HighPriorityISR (void)
{
	static BOOL hpTest;

	hpTest = FALSE;

	if (IPR1bits.TMR2IP && PIR1bits.TMR2IF)
	{
		Timer2Isr();
		hpTest = TRUE;
	}
	if (IPR1bits.TXIP && PIR1bits.TXIF)
	{
		SerialTxIsr();
		hpTest = TRUE;
	}

	if (IPR1bits.RCIP && PIR1bits.RCIF)
	{
		SerialRxIsr();
		hpTest = TRUE;
	}

	// Throw a warning if we don't know why were interrupted.

	if (!hpTest)
		throwWarning(WARN_ISR_UNKNOWN_HPI);
}



/*---------------------------------------------------------------------------\
| Function:		LowPriorityISR
|
| Purpose:		Handle all of the low prioirty interrupts
|
| Discussion:	Checks to see what interrupted us.
|
| Expects:		Low prioirty interrupts disabled but high prioirty may still
|				be enabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE LOW PRIOIRTY INTERRUPT SERVICE ROUTINE
// USING THE #PRAGMA DIRECTIVE (ugly, but true).

#pragma interruptlow LowPriorityISR

void LowPriorityISR (void)
{
	static BOOL lpTest;

	lpTest = FALSE;

	if (!IPR1bits.TMR2IP && PIR1bits.TMR2IF)
	{
		Timer2Isr();
		lpTest = TRUE;
	}
	if (!IPR1bits.TXIP && PIR1bits.TXIF)
	{
		SerialTxIsr();
		lpTest = TRUE;
	}

	if (!IPR1bits.RCIP && PIR1bits.RCIF)
	{
		SerialRxIsr();
		lpTest = TRUE;
	}

	// Throw a warning if we don't know why were interrupted.

	if (!lpTest)
		throwWarning(WARN_ISR_UNKNOWN_LPI);

}




/*---------------------------------------------------------------------------\
| Function:		InitializeInterrupts
|
| Purpose:		Initialize the PIC's interrupts
|
| Discussion:	Huh?
|
| Expects:		All interupts to be off (after all, we were just reseted).
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializeInterrupts (void)
{
	// Turn on "high and low" priorities, not old peripheral/global

	EnablePrioritizedInts();

	// Turn on ints and away we go!

	EnableHighPriorityInts();
	EnableLowPriorityInts();
}

